<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Cus02700CustoPadraoXRealizado  extends \Zeedhi\Framework\Controller\Report {
    protected $novoCodigo;
    protected $procedureConsumoPlanejado;
    protected $consplan;
    protected $nrorg;
    protected $cdfilial;
    protected $nrConsPlan;
    protected $birtStrategy;
    protected $environment;
    protected $connection;
    protected $crudService;
    protected $dataSourceManager;
    
    public function __construct($reportService, $novoCodigo, $consplan, $birtStrategy, $environment, $util, StoredProcedure $procedureConsplanGeral, $connection, $crudService, $dataSourceManager) {
      $this->novoCodigo                 = $novoCodigo;
      $this->consplan                   = $consplan;
      $this->reportService              = $reportService;
      $this->birtStrategy               = $birtStrategy;
      $this->environment                = $environment;
      $this->util                       = $util;
      $this->procedureConsplanGeral     = $procedureConsplanGeral;
      $this->connection                 = $connection;
      $this->crudService                = $crudService;
      $this->dataSourceManager          = $dataSourceManager;
  }

    protected $reportMapping = array (
      'CUS02700' => 
            array (
              'strategy' => 'BIRT',
              'parameterMapping' => 
              array (
                "P_NRORG"           => "P_NRORG",
                "CDCALCULABASEADTA" => "P_TPDTCUS",
                "LSERVICO"          => "L_SERVICO",
                "DTINICIAL"         => "P_DTINICIAL",
                "DTFINAL"           => "P_DTFINAL",
                "NMTIPOCUSTOPLAN2"   => "P_NRTIPOCUSTOPLAN",
                "DTMESINIC"         => "P_DTMESINIC",
                "DTMESFINA"         => "P_DTMESFINA",
                "NRCONSPLAN"        => "P_NRCONSPLAN",
                "NMTIPOCUSTOREAL2"   => "P_NRTIPOCUSTOREAL",
                "NMTIPOCUSTOPLAN"   => "P_NMTIPOCUSTOPLAN",
                "NMTIPOCUSTOREAL"   => "P_NMTIPOCUSTOREAL"
              ),
            ),
      'CUS02701' => 
            array (
              'strategy' => 'BIRT',
              'parameterMapping' => 
              array (
                "P_NRORG"           => "P_NRORG",
                "CDCALCULABASEADTA" => "P_TPDTCUS",
                "LSERVICO"          => "L_SERVICO",
                "DTINICIAL"         => "P_DTINICIAL",
                "DTFINAL"           => "P_DTFINAL",
                "NMTIPOCUSTOPLAN2"   => "P_NRTIPOCUSTOPLAN",
                "DTMESINIC"         => "P_DTMESINIC",
                "DTMESFINA"         => "P_DTMESFINA",
                "NRCONSPLAN"        => "P_NRCONSPLAN",
                "NMTIPOCUSTOREAL2"   => "P_NRTIPOCUSTOREAL",
                "NMTIPOCUSTOPLAN"   => "P_NMTIPOCUSTOPLAN",
                "NMTIPOCUSTOREAL"   => "P_NMTIPOCUSTOREAL"
              ),
            ),   
    );

  public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
    $row = $request->getRow();
    
    $cdOperador = $this->environment->getCdOperador();
    
    $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
    $row['P_NRORG'] = $this->environment->getNrorg();
    $servico =  '';
    if(count($row['NMSERVICO']) == 0){
      $listaserv = $this->getServicos($row);
      $row['LSERVICO'] = $this->util->insertLista($listaserv);
      $servico =  $this->util->insertLista($listaserv);
    }else{
      $row['LSERVICO'] = $this->util->insertLista($row['NMSERVICO']);
      $servico =  $this->util->insertLista($row['NMSERVICO']);
    }
    $procedure = new StoredProcedure($this->connection, 'CONSPLAN_GERAL');
    $procedure->addParam(new Param('P_NRLISTA',      Param::PARAM_INPUT,  $servico,               Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT,  $row['DTINICIAL'],      Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT,  $row['DTFINAL'],        Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT,  'CFP',                  Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT,  'N',                    Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT,  0,                      Param::PARAM_TYPE_INT));
    $procedure->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT,  'N',                    Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT,  '0',                    Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT,  'zzzzzzzzzz',           Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT,  $cdOperador,            Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_NRCONSPLANTH', Param::PARAM_INPUT,  'GERANOVCOD',           Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_OUTPUT, null,                   Param::PARAM_TYPE_STR, 10));
    
    $procedureResult = $procedure->execute();
    $row['NRCONSPLAN'] = $procedureResult["P_NRCONSPLAN"];
  }
  
  private function getFiliais($row) {
    $filiais = array();
    $grupo = 'T';
    if(isset($row['CDGRUPFILI'])){
      if(!empty($row['CDGRUPFILI'])){
        $grupo = $row['CDGRUPFILI'];
      }
    }
    $conditions = array(
        array(
            'columnName' => "CDGRUPFILI",
            'operator' => '=',
            'value' => $grupo
        ),
        array(
            'columnName' => "CDOPERADOR",
            'operator' => '=',
            'value' => $this->crudService->getEnvironment()->getCdOperador()
        ),
        array(
            'columnName' => "NRORG",
            'operator' => '=',
            'value' => $this->crudService->getEnvironment()->getNrOrg()
        )
    );
    $filterCriteria = new FilterCriteria('filial_operador_grupfili', $conditions);
    $filterCriteria = $this->crudService->populateParameterBag($filterCriteria, array("CDGRUPFILI", "CDOPERADOR", "NRORG"));
    $dataSetFiliais = $this->dataSourceManager->findBy($filterCriteria)->getRows();
    foreach ($dataSetFiliais as $filial) {
        array_push($filiais, $filial['CDFILIAL']);
    }
    return $filiais;
  }
  
  private function getServicos($row) {
    $servicos = array();
    $grupo = 'T';
    if(isset($row['CDGRUPFILI'])){
      if(!empty($row['CDGRUPFILI'])){
        $grupo = $row['CDGRUPFILI'];
      }
    }
    $conditions = array(
        array(
            'columnName' => "CDFILIAL",
            'operator' => '=',
            'value' => $row['LISTAFILIAL']
        ),
        array(
            'columnName' => "CDGRUPFILI",
            'operator' => '=',
            'value' => $grupo
        ),
        array(
            'columnName' => "CDOPERADOR",
            'operator' => '=',
            'value' => $this->crudService->getEnvironment()->getCdOperador()
        ),
        array(
            'columnName' => "NRORG",
            'operator' => '=',
            'value' => $this->crudService->getEnvironment()->getNrOrg()
        )
    );
    $filterCriteria = new FilterCriteria('servicos_por_agrupfili_multi_filiais', $conditions);
    $filterCriteria = $this->crudService->populateParameterBag($filterCriteria, array("CDFILIAL", "CDGRUPFILI", "CDOPERADOR", "NRORG"));
    $dataSetservico = $this->dataSourceManager->findBy($filterCriteria)->getRows();
    foreach ($dataSetservico as $servico) {
        array_push($servicos, $servico['FILISERV']);
    }
    return $servicos;
  }
}